@@ -16,8 +16,6 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <config.h>
-
 #include <glom/bakery/app_withdoc_gtk.h>
 #include <glom/bakery/dialog_offersave.h>
 //#include <libgnomevfsmm/utils.h> //For escape_path_string()
@@ -30,13 +28,18 @@
 #include <gtkmm/aboutdialog.h>
 #include <giomm.h>
 #include <algorithm>
+
+#include <config.h>
+
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildon-fmmm/file-chooser-dialog.h>
+#include <hildon/hildon-window.h>
+#endif // GLOM_ENABLE_MAEMO
+
 #include <glibmm/i18n-lib.h>
 
 //#include <gtk/gtkfilesel.h>
 
-
-#undef GLOM_ENABLE_MAEMO
-
 namespace GlomBakery
 {
 
@@ -65,6 +68,11 @@ App_WithDoc_Gtk::App_WithDoc_Gtk(BaseObjectType* cobject, const Glib::ustring& a
 {
   init_app_name(appname);
 
+  #ifdef GLOM_ENABLE_MAEMO
+  //The .glade file needs to specify HildonWindow or features such as HildonAppMenu won't work.
+  g_assert(HILDON_IS_WINDOW(gobj()));
+  #endif //GLOM_ENABLE_MAEMO
+
 #ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   signal_hide().connect(sigc::mem_fun(*this, &App_WithDoc_Gtk::on_hide));
   signal_delete_event().connect(sigc::mem_fun(*this, &App_WithDoc_Gtk::on_delete_event));
@@ -119,9 +127,10 @@ void App_WithDoc_Gtk::init_layout()
 
   //Add menu bar at the top:
   //These were defined in init_uimanager().
-#ifdef GLOM_ENABLE_MAEMO //TODO: Use Hildon::AppMenu
-  Gtk::Menu* pMenu = static_cast<Gtk::Menu*>(m_refUIManager->get_widget("/Bakery_MainMenu"));
-  set_menu(*pMenu);
+#ifdef GLOM_ENABLE_MAEMO
+  //TODO: Use Hildon::AppMenu
+  //Gtk::Menu* pMenu = static_cast<Gtk::Menu*>(m_refUIManager->get_widget("/Bakery_MainMenu"));
+  //set_menu(*pMenu);
 #else
   Gtk::MenuBar* pMenuBar = static_cast<Gtk::MenuBar*>(m_refUIManager->get_widget("/Bakery_MainMenu"));
   m_pVBox->pack_start(*pMenuBar, Gtk::PACK_SHRINK);
@@ -468,22 +477,22 @@ void App_WithDoc_Gtk::update_window_title()
 {
   //Set application's main window title:
 
-  Glib::ustring strTitle = m_strAppName;
   Document* pDoc = get_document();
-  if(pDoc)
-  {
-    strTitle += " - " + pDoc->get_name();
+  if(!pDoc)
+    return;
 
-    //Indicate unsaved changes:
-    if(pDoc->get_modified())
-      strTitle += " *";
+  Glib::ustring strTitle = m_strAppName;
+  strTitle += " - " + pDoc->get_name();
 
-    //Indicate read-only files:
-    if(pDoc->get_read_only())
-      strTitle += _(" (read-only)");
+  //Indicate unsaved changes:
+  if(pDoc->get_modified())
+    strTitle += " *";
 
-    set_title(strTitle);
-  }
+  //Indicate read-only files:
+  if(pDoc->get_read_only())
+    strTitle += _(" (read-only)");
+
+  set_title(strTitle);
 }
 
 void App_WithDoc_Gtk::ui_warning(const Glib::ustring& text, const Glib::ustring& secondary_text)
@@ -491,7 +500,7 @@ void App_WithDoc_Gtk::ui_warning(const Glib::ustring& text, const Glib::ustring&
   Gtk::Window* pWindow = this;
 
 #ifdef GLOM_ENABLE_MAEMO
-  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, text, Gtk::Stock::DIALOG_WARNING);
+  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, *pWindow, text);
 #else
   Gtk::MessageDialog dialog(App_WithDoc_Gtk::util_bold_message(text), true /* use markup */, Gtk::MESSAGE_WARNING);
   dialog.set_secondary_text(secondary_text);
@@ -606,25 +615,11 @@ Glib::ustring App_WithDoc_Gtk::ui_file_select_save(const Glib::ustring& old_file
     }
   }
 
-
-  //bool tried_once_already = false;
-
   bool try_again = true;
   while(try_again)
   {
     try_again = false;
 
-    //Work around bug #330680 "GtkFileChooserDialog is too small when shown a second time.":
-    //(Commented-out because the workaround doesn't work)
-    /*
-    if(tried_once_already)
-    {
-      fileChooser_Save.set_default_size(-1, 600); 
-    }
-    else
-      tried_once_already = true;
-    */
-
     const int response_id = fileChooser_Save.run();
     fileChooser_Save.hide();
     if(response_id != Gtk::RESPONSE_CANCEL)
