@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libglom/libglom_config.h>
+#include "config.h"
 #include "dialog_existing_or_new.h"
 
 #include <libxml++/parsers/saxparser.h>
@@ -30,6 +30,10 @@
 #include <gtkmm/filechooserdialog.h>
 #include <gtkmm/stock.h>
 
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildon-fmmm/file-chooser-dialog.h>
+#endif // GLOM_ENABLE_MAEMO
+
 #ifdef G_OS_WIN32
 # include <glib/gwin32.h>
 #else
@@ -46,9 +50,12 @@ namespace
 {
 
 const char* RECENT_DUMMY_TEXT = N_("No recently used documents available.");
-const char* TEMPLATE_DUMMY_TEXT = N_("No templates available.");
 const char* NETWORK_DUMMY_TEXT = N_("No sessions found on the local network.");
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+const char* TEMPLATE_DUMMY_TEXT = N_("No templates available.");
+#endif
+
 //TODO_Performance: A DomParser or XmlReader might be faster, or even a regex.
 /// Reads the title of an example from the first few characters of the XML.
 class Parser: public xmlpp::SaxParser
@@ -94,46 +101,31 @@ private:
 namespace Glom
 {
 
-Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
+Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Gtk::Dialog(cobject)
 {
-  refGlade->get_widget("existing_or_new_existing_treeview", m_existing_view);
-  refGlade->get_widget("existing_or_new_new_treeview", m_new_view);
+#ifdef GLOM_ENABLE_CLIENT_ONLY
+  //Don't mention creation of new documents in client-only mode:
+  Gtk::Label* label = 0;
+  builder->get_widget("existing_or_new_label", label);
+  label->set_text(_("Open a Document"));
+#endif //GLOM_ENABLE_CLIENT_ONLY
+  
+  builder->get_widget("existing_or_new_existing_treeview", m_existing_view);
+  g_assert(m_existing_view);
 
-  if(!m_existing_view || !m_new_view)
-    throw std::runtime_error("Glade file does not contain treeviews for ExistingOrNew dialog.");
-
-  refGlade->get_widget("existing_or_new_notebook", m_notebook);
-  refGlade->get_widget("existing_or_new_button_select", m_select_button);
+  builder->get_widget("existing_or_new_notebook", m_notebook);
+  builder->get_widget("existing_or_new_button_select", m_select_button);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   if(!m_notebook || !m_select_button)
     throw std::runtime_error("Glade file does not contain the notebook or the select button for ExistingOrNew dialog.");
+#endif
 
   m_existing_model = Gtk::TreeStore::create(m_existing_columns);
   m_existing_model->set_sort_column(m_existing_columns.m_col_time, Gtk::SORT_DESCENDING);
   m_existing_view->set_model(m_existing_model);
 
-  m_new_model = Gtk::TreeStore::create(m_new_columns);
-  m_new_view->set_model(m_new_model);
-
-  m_existing_column_title.set_expand(true);
-  m_existing_column_title.pack_start(m_existing_icon_renderer, false);
-  m_existing_column_title.pack_start(m_existing_title_renderer, true);
-  m_existing_column_title.set_cell_data_func(m_existing_icon_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::existing_icon_data_func));
-  m_existing_column_title.set_cell_data_func(m_existing_title_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::existing_title_data_func));
-  m_existing_view->append_column(m_existing_column_title);
-
-
-  m_new_column_title.set_expand(true);
-  m_new_column_title.pack_start(m_new_icon_renderer, false);
-  m_new_column_title.pack_start(m_new_title_renderer, true);
-  m_new_column_title.set_cell_data_func(m_new_icon_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_icon_data_func));
-  m_new_column_title.set_cell_data_func(m_new_title_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_title_data_func));
-  m_new_view->append_column(m_new_column_title);
-
-  m_existing_view->set_headers_visible(false);
-  m_existing_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_row_activated));
-  m_new_view->set_headers_visible(false);
-  m_new_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_row_activated));
+ 
 
   m_iter_existing_other = m_existing_model->append();
   (*m_iter_existing_other)[m_existing_columns.m_col_title] = _("Select File");
@@ -146,36 +138,35 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   m_iter_existing_recent = m_existing_model->append();
   (*m_iter_existing_recent)[m_existing_columns.m_col_title] = _("Recently Opened");
   
-  m_iter_new_empty = m_new_model->append();
-  (*m_iter_new_empty)[m_new_columns.m_col_title] = _("New Empty Document");
-
-  m_iter_new_template = m_new_model->append();
-  (*m_iter_new_template)[m_new_columns.m_col_title] = _("New From Template");
+ 
 
+  m_existing_column_title.set_expand(true);
+  m_existing_column_title.pack_start(m_existing_icon_renderer, false);
+  m_existing_column_title.pack_start(m_existing_title_renderer, true);
+  m_existing_column_title.set_cell_data_func(m_existing_icon_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::existing_icon_data_func));
+  m_existing_column_title.set_cell_data_func(m_existing_title_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::existing_title_data_func));
+  m_existing_view->append_column(m_existing_column_title);
 
+  m_existing_view->set_headers_visible(false);
+  m_existing_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_row_activated));
+ 
  // Load example files:
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 
 #ifdef G_OS_WIN32
-  gchar* dir = g_win32_get_package_installation_subdirectory(NULL, NULL, "share/glom/doc/examples");
-  std::string path(dir);
+  gchar* dir = g_win32_get_package_installation_directory_of_module(0);
+  std::string path = Glib::build_filename(dir, "share" G_DIR_SEPARATOR_S "doc"
+                                                       G_DIR_SEPARATOR_S "glom"
+                                                       G_DIR_SEPARATOR_S "examples");
   g_free(dir);
 
   if(!Glib::file_test(path, Glib::FILE_TEST_EXISTS))
-    path = GLOM_EXAMPLES_DIR;
+    path = GLOM_DOCDIR G_DIR_SEPARATOR_S "examples";
 #else
-  const char* path = GLOM_EXAMPLES_DIR;
+  const char *const path = GLOM_DOCDIR G_DIR_SEPARATOR_S "examples";
 #endif //G_OS_WIN32
 
-  if(!list_examples_at_path(path))
-  {
-    // If that directory did not exist, then try this one instead:
-    // (An Ubuntu package puts the example files here for some reason.
-    // TODO: Find out what Makefile.am variable to use to just use this automatically instead.
-    #ifndef G_OS_WIN32
-    list_examples_at_path(GLOM_EXAMPLES_DIR_ALTERNATIVE);
-    #endif //G_OS_WIN32
-  }
+  list_examples_at_path(path);
 
 #endif //!GLOM_ENABLE_CLIENT_ONLY
 
@@ -199,7 +190,7 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
       Gtk::TreeModel::iterator iter = m_existing_model->append(m_iter_existing_recent->children());
       (*iter)[m_existing_columns.m_col_title] = info->get_display_name();
       (*iter)[m_existing_columns.m_col_time] = info->get_modified();
-      (*iter)[m_existing_columns.m_col_recent_info] = new Glib::RefPtr<Gtk::RecentInfo>(info);
+      (*iter)[m_existing_columns.m_col_recent_info] = info;
     }
   }
 
@@ -212,10 +203,11 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   m_iter_existing_network_dummy = create_dummy_item_existing(m_iter_existing_network, _(NETWORK_DUMMY_TEXT));
 #endif
 
-  m_iter_new_template_dummy = create_dummy_item_new(m_iter_new_template, _(TEMPLATE_DUMMY_TEXT));
-
-  // Expand recently used files
+ 
+  // Expand recently used files and the networked files,
+  // because the contents help to explain what this is: 
   m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_recent), false);
+  m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_network), false);
 
   m_select_button->signal_clicked().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_select_clicked));
   m_select_button->set_image(*Gtk::manage(new Gtk::Image(Gtk::Stock::APPLY, Gtk::ICON_SIZE_BUTTON)));
@@ -229,6 +221,29 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   existing_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_select_func) ); 
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+  builder->get_widget("existing_or_new_new_treeview", m_new_view);
+  g_assert(m_new_view);
+  m_new_model = Gtk::TreeStore::create(m_new_columns);
+  m_new_view->set_model(m_new_model);
+  
+  m_iter_new_empty = m_new_model->append();
+  (*m_iter_new_empty)[m_new_columns.m_col_title] = _("New Empty Document");
+
+  m_iter_new_template = m_new_model->append();
+  (*m_iter_new_template)[m_new_columns.m_col_title] = _("New From Template");
+  
+  m_new_column_title.set_expand(true);
+  m_new_column_title.pack_start(m_new_icon_renderer, false);
+  m_new_column_title.pack_start(m_new_title_renderer, true);
+  m_new_column_title.set_cell_data_func(m_new_icon_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_icon_data_func));
+  m_new_column_title.set_cell_data_func(m_new_title_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_title_data_func));
+  m_new_view->append_column(m_new_column_title);
+  
+  m_new_view->set_headers_visible(false);
+  m_new_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_row_activated));
+
+  m_iter_new_template_dummy = create_dummy_item_new(m_iter_new_template, _(TEMPLATE_DUMMY_TEXT));
+ 
   Glib::RefPtr<Gtk::TreeView::Selection> new_view_selection = m_new_view->get_selection();
   new_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_selection_changed));
   new_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_select_func) );
@@ -237,13 +252,18 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   m_notebook->set_show_tabs(false);
 #endif /* !GLOM_ENABLE_CLIENT_ONLY */
     
+  //Make sure the first item is visible, 
+  //which is not always the case on Maemo. 
+  m_existing_view->scroll_to_row( 
+    Gtk::TreeModel::Path(m_iter_existing_other) );
+
   update_ui_sensitivity();
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 bool Dialog_ExistingOrNew::list_examples_at_path(const std::string& path)
 {
-  std::cout << "Dialog_ExistingOrNew::list_examples_at_path(): path=" << path << std::endl;
+  //std::cout << "Debug: Dialog_ExistingOrNew::list_examples_at_path(): path=" << path << std::endl;
 
   m_examples_dir = Gio::File::create_for_path(path);
 
@@ -280,17 +300,6 @@ Dialog_ExistingOrNew::~Dialog_ExistingOrNew()
   }
 #endif
 
-  // Release the recent infos (see comment in the header for why these
-  // have to be dynamically allocated)
-  if(!m_iter_existing_recent_dummy.get())
-  {
-    const Gtk::TreeNodeChildren& children = m_iter_existing_recent->children();
-    for(Gtk::TreeModel::iterator iter = children.begin(); iter != children.end(); ++ iter)
-    {
-      Glib::RefPtr<Gtk::RecentInfo>* info = (*iter)[m_existing_columns.m_col_recent_info];
-      delete info;
-    }
-  }
 }
 
 bool Dialog_ExistingOrNew::on_existing_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool /* path_currently_selected */)
@@ -306,6 +315,7 @@ bool Dialog_ExistingOrNew::on_existing_select_func(const Glib::RefPtr<Gtk::TreeM
   return true;
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 bool Dialog_ExistingOrNew::on_new_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool /* path_currently_selected */)
 {
   Gtk::TreeModel::iterator iter = model->get_iter(path);
@@ -314,7 +324,7 @@ bool Dialog_ExistingOrNew::on_new_select_func(const Glib::RefPtr<Gtk::TreeModel>
   else
     return true;
 }
-
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
 Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action_impl(Gtk::TreeModel::iterator& iter) const
 {
@@ -337,6 +347,7 @@ Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action_impl(Gtk::TreeMode
   }
   else
   {
+    #ifndef GLOM_ENABLE_CLIENT_ONLY
     if(m_new_view->get_selection()->count_selected_rows() == 0)
       return NONE;
 
@@ -347,7 +358,10 @@ Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action_impl(Gtk::TreeMode
       return NEW_EMPTY;
     else
       return NONE;
+    #endif //GLOM_ENABLE_CLIENT_ONLY
   }
+  
+  return NONE;
 }
 
 Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action() const
@@ -359,13 +373,16 @@ Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action() const
 Glib::ustring Dialog_ExistingOrNew::get_uri() const
 {
   Gtk::TreeModel::iterator iter;
-  Action action = get_action_impl(iter);
+  const Action action = get_action_impl(iter);
 
+  #ifndef GLOM_ENABLE_CLIENT_ONLY
   if(action == NEW_FROM_TEMPLATE)
   {
     return (*iter)[m_new_columns.m_col_template_uri];
   }
-  else if(action == OPEN_URI)
+  else 
+  #endif //GLOM_ENABLE_CLIENT_ONLY
+  if(action == OPEN_URI)
   {
     if(iter == m_iter_existing_other)
     {
@@ -373,14 +390,18 @@ Glib::ustring Dialog_ExistingOrNew::get_uri() const
     }
     else
     {
-      Glib::RefPtr<Gtk::RecentInfo>* info = (*iter)[m_existing_columns.m_col_recent_info];
-      return (*info)->get_uri();
+      Glib::RefPtr<Gtk::RecentInfo> info = (*iter)[m_existing_columns.m_col_recent_info];
+      return info->get_uri();
     }
   }
   else
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     throw std::logic_error("Dialog_ExistingOrNew::get_uri: action is neither NEW_FROM_TEMPLATE nor OPEN_URI");
+#endif    
   }
+
+  return Glib::ustring();
 }
 
 #ifndef G_OS_WIN32
@@ -391,8 +412,12 @@ EpcServiceInfo* Dialog_ExistingOrNew::get_service_info() const
 
   if(action == OPEN_REMOTE)
     return (*iter)[m_existing_columns.m_col_service_info];
+#ifdef GLIBMM_EXCEPTIONS_ENABLED    
   else
     throw std::logic_error("Dialog_ExistingOrNew::get_service_info: action is not OPEN_REMOTE");
+#endif
+
+  return 0;
 }
 
 Glib::ustring Dialog_ExistingOrNew::get_service_name() const
@@ -402,8 +427,12 @@ Glib::ustring Dialog_ExistingOrNew::get_service_name() const
 
   if(action == OPEN_REMOTE)
     return (*iter)[m_existing_columns.m_col_service_name];
+#ifdef GLIBMM_EXCEPTIONS_ENABLED    
   else
     throw std::logic_error("Dialog_ExistingOrNew::get_service_name: action is not OPEN_REMOTE");
+#endif
+
+  return Glib::ustring();
 }
 #endif
 
@@ -414,41 +443,72 @@ std::auto_ptr<Gtk::TreeModel::iterator> Dialog_ExistingOrNew::create_dummy_item_
   return std::auto_ptr<Gtk::TreeModel::iterator>(new Gtk::TreeModel::iterator(iter));
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 std::auto_ptr<Gtk::TreeModel::iterator> Dialog_ExistingOrNew::create_dummy_item_new(const Gtk::TreeModel::iterator& parent, const Glib::ustring& text)
 {
   Gtk::TreeModel::iterator iter = m_new_model->append(parent->children());
   (*iter)[m_new_columns.m_col_title] = text;
   return std::auto_ptr<Gtk::TreeModel::iterator>(new Gtk::TreeModel::iterator(iter));
 }
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
 void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
-  Gtk::CellRendererPixbuf* pixbuf_renderer = dynamic_cast<Gtk::CellRendererPixbuf*>(renderer);
-  if(!pixbuf_renderer) throw std::logic_error("Renderer not a pixbuf renderer in existing_icon_data_func");
+  Gtk::CellRendererPixbuf* pixbuf_renderer = dynamic_cast<Gtk::CellRendererPixbuf*>(renderer);  
+  if(!pixbuf_renderer) 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  throw std::logic_error("Renderer not a pixbuf renderer in existing_icon_data_func");
+#else
+  return;
+#endif
 
+#ifdef GLIBMM_PROPERTIES_ENABKED
   pixbuf_renderer->property_stock_size() = Gtk::ICON_SIZE_BUTTON;
   pixbuf_renderer->property_stock_id() = "";
   pixbuf_renderer->property_pixbuf() = Glib::RefPtr<Gdk::Pixbuf>();
 
   if(iter == m_iter_existing_recent)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::INDEX.id; // TODO: More meaningful icon?
+    pixbuf_renderer->property_stock_id() = Gtk::StockID(Gtk::Stock::INDEX); // TODO: More meaningful icon?
 #ifndef G_OS_WIN32
   else if(iter == m_iter_existing_network)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::NETWORK.id;
+    pixbuf_renderer->property_stock_id() = Gtk::StockID(Gtk::Stock::NETWORK);
 #endif
   else if(iter == m_iter_existing_other)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::OPEN.id;
-  else if(m_iter_existing_recent_dummy.get() != NULL && iter == *m_iter_existing_recent_dummy)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::DIALOG_ERROR.id; // TODO: Use Stock::STOP instead?
+    pixbuf_renderer->property_stock_id() = Gtk::StockID(Gtk::Stock::OPEN);
+  else if(m_iter_existing_recent_dummy.get() && iter == *m_iter_existing_recent_dummy)
+    pixbuf_renderer->property_stock_id() = Gtk::StockID(Gtk::Stock::DIALOG_ERROR); // TODO: Use Stock::STOP instead?
 #ifndef G_OS_WIN32
-  else if(m_iter_existing_network_dummy.get() != NULL && iter == *m_iter_existing_network_dummy)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::DIALOG_ERROR.id; // TODO: Use Stock::STOP instead?
+  else if(m_iter_existing_network_dummy.get() && iter == *m_iter_existing_network_dummy)
+    pixbuf_renderer->property_stock_id() = Gtk::StockID(Gtk::Stock::DIALOG_ERROR); // TODO: Use Stock::STOP instead?
 #endif
+
+#else // Properties enabled
+
+  pixbuf_renderer->set_property("stock-size", Gtk::ICON_SIZE_BUTTON);
+  pixbuf_renderer->set_property("stock-id", std::string());
+  pixbuf_renderer->set_property("pixbuf", Glib::RefPtr<Gdk::Pixbuf>());
+
+  if(iter == m_iter_existing_recent)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::INDEX));
+#ifndef G_OS_WIN32
+  else if(iter == m_iter_existing_network)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::NETWORK));
+#endif
+  else if(iter == m_iter_existing_other)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::OPEN));
+  else if(m_iter_existing_recent_dummy.get() && iter == *m_iter_existing_recent_dummy)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::DIALOG_ERROR)); // TODO: Use Stock::STOP instead?
+#ifndef G_OS_WIN32
+  else if(m_iter_existing_network_dummy.get() && iter == *m_iter_existing_network_dummy)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::DIALOG_ERROR)); // TODO: Use Stock::STOP instead?
+#endif
+
+#endif // Properties enabled
   else
   {
     if(m_existing_model->is_ancestor(m_iter_existing_recent, iter))
     {
-      //Glib::RefPtr<Gtk::RecentInfo>* info = (*iter)[m_existing_columns.m_col_recent_info];
+      //Glib::RefPtr<Gtk::RecentInfo> info = (*iter)[m_existing_columns.m_col_recent_info];
       //pixbuf_renderer->property_pixbuf() = (*info)->get_icon(Gtk::ICON_SIZE_BUTTON);
       pixbuf_renderer->set_property("icon-name", Glib::ustring("glom"));
     }
@@ -461,7 +521,9 @@ void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer,
 #endif
     else
     {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       throw std::logic_error("Unexpected iterator in existing_icon_data_func");
+#endif      
     }
   }
 }
@@ -469,13 +531,20 @@ void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer,
 void Dialog_ExistingOrNew::existing_title_data_func(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
   Gtk::CellRendererText* text_renderer = dynamic_cast<Gtk::CellRendererText*>(renderer);
-  if(!text_renderer) throw std::logic_error("Renderer not a text renderer in existing_title_data_func");
+  if(!text_renderer)
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+ throw std::logic_error("Renderer not a text renderer in existing_title_data_func");
+#else
+  return;
+#endif    
 
+#ifdef GLIBMM_PROPERTIES_ENABLED
   text_renderer->property_text() = (*iter)[m_existing_columns.m_col_title];
 
-  // Default: Use default color
+  // Default: Use default color:
   text_renderer->property_foreground_set() = false;
-  // Use grey if parent item has no children
+  
+  // Use grey if parent item has no children:
 #ifndef G_OS_WIN32
   if( (iter == m_iter_existing_network && m_iter_existing_network_dummy.get()) ||
       (iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get()))
@@ -485,13 +554,36 @@ void Dialog_ExistingOrNew::existing_title_data_func(Gtk::CellRenderer* renderer,
   {
     text_renderer->property_foreground() = "grey";
   }
+#else // PROPERTIES_ENABLED
+  Glib::ustring text = (*iter)[m_existing_columns.m_col_title];
+  text_renderer->set_property("text", text);
+  // Default: Use default color
+  text_renderer->set_property("foreground-set", false);
+  // Use grey if parent item has no children
+#ifndef G_OS_WIN32
+  if( (iter == m_iter_existing_network && m_iter_existing_network_dummy.get()) ||
+      (iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get()))
+#else
+  if(iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get())
+#endif
+  {
+    text_renderer->set_property("foreground", std::string("grey"));
+  } 
+#endif // PROPERTIES_ENABLED
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void Dialog_ExistingOrNew::new_icon_data_func(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
   Gtk::CellRendererPixbuf* pixbuf_renderer = dynamic_cast<Gtk::CellRendererPixbuf*>(renderer);
-  if(!pixbuf_renderer) throw std::logic_error("Renderer not a pixbuf renderer in new_icon_data_func");
+  if(!pixbuf_renderer)
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  throw std::logic_error("Renderer not a pixbuf renderer in new_icon_data_func");
+#else
+  return;
+#endif
 
+#ifdef GLIBMM_PROPERTIES_ENABLED
   pixbuf_renderer->property_stock_size() = Gtk::ICON_SIZE_BUTTON;
   pixbuf_renderer->property_stock_id() = "";
   pixbuf_renderer->property_pixbuf() = Glib::RefPtr<Gdk::Pixbuf>();
@@ -500,7 +592,7 @@ void Dialog_ExistingOrNew::new_icon_data_func(Gtk::CellRenderer* renderer, const
     pixbuf_renderer->property_stock_id() = Gtk::Stock::NEW.id;
   else if(iter == m_iter_new_template)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::EDIT.id; // TODO: More meaningful icon?
-  else if(m_iter_new_template_dummy.get() != NULL && iter == *m_iter_new_template_dummy)
+  else if(m_iter_new_template_dummy.get() && iter == *m_iter_new_template_dummy)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::DIALOG_ERROR.id; // TODO: Use Stock::STOP instead?
   else
   {
@@ -510,16 +602,49 @@ void Dialog_ExistingOrNew::new_icon_data_func(Gtk::CellRenderer* renderer, const
     }
     else
     {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED    
       throw std::logic_error("Unexpected iterator in new_icon_data_func");
+#endif      
     }
   }
+#else
+  pixbuf_renderer->set_property("stock-size", Gtk::ICON_SIZE_BUTTON);
+  pixbuf_renderer->set_property("stock-id", Glib::ustring());
+  pixbuf_renderer->set_property("pixbuf", Glib::RefPtr<Gdk::Pixbuf>());
+      
+  if(iter == m_iter_new_empty)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::NEW));
+  else if(iter == m_iter_new_template)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::EDIT)); // TODO: More meaningful icon?
+  else if(m_iter_new_template_dummy.get() && iter == *m_iter_new_template_dummy)
+    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::DIALOG_ERROR)); // TODO: Use Stock::STOP instead?
+  else
+  {
+    if(m_new_model->is_ancestor(m_iter_new_template, iter))
+    {
+      pixbuf_renderer->set_property("icon-name", Glib::ustring("glom"));
+    }
+    else
+    {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED    
+      throw std::logic_error("Unexpected iterator in new_icon_data_func");
+#endif      
+    }
+  }
+#endif 
 }
 
 void Dialog_ExistingOrNew::new_title_data_func(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
   Gtk::CellRendererText* text_renderer = dynamic_cast<Gtk::CellRendererText*>(renderer);
-  if(!text_renderer) throw std::logic_error("Renderer not a text renderer in new_title_data_func");
+  if(!text_renderer) 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    throw std::logic_error("Renderer not a text renderer in new_title_data_func");
+#else
+    return;
+#endif    
 
+#ifdef GLIBMM_PROPERTIES_ENABLED
   text_renderer->property_text() = (*iter)[m_new_columns.m_col_title];
 
   // Default: Use default color
@@ -529,7 +654,20 @@ void Dialog_ExistingOrNew::new_title_data_func(Gtk::CellRenderer* renderer, cons
   {
     text_renderer->property_foreground() = "grey";
   }
+#else
+  Glib::ustring text = (*iter)[m_new_columns.m_col_title];
+  text_renderer->set_property("text", text);
+
+  // Default: Use default color
+  text_renderer->set_property("foreground-set", false);
+  // Use grey if parent item has no children
+  if( (iter == m_iter_new_template && m_iter_new_template_dummy.get()))
+  {
+    text_renderer->set_property("foreground", Glib::ustring("grey"));
+  }
+#endif    
 }
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
 void Dialog_ExistingOrNew::on_switch_page(GtkNotebookPage* /* page */, guint /* page_num */)
 {
@@ -541,10 +679,12 @@ void Dialog_ExistingOrNew::on_existing_selection_changed()
   update_ui_sensitivity();
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void Dialog_ExistingOrNew::on_new_selection_changed()
 {
   update_ui_sensitivity();
 }
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
 void Dialog_ExistingOrNew::update_ui_sensitivity()
 {
@@ -572,6 +712,7 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
 #endif
     }
   }
+  #ifndef GLOM_ENABLE_CLIENT_ONLY
   else
   {
     const int count = m_new_view->get_selection()->count_selected_rows();
@@ -587,6 +728,7 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
                      (!m_iter_new_template_dummy.get() || sel != *m_iter_new_template_dummy));
     }
   }
+  #endif //GLOM_ENABLE_CLIENT_ONLY
 
   m_select_button->set_sensitive(sensitivity);
 }
@@ -686,7 +828,7 @@ void Dialog_ExistingOrNew::on_stream_read(const Glib::RefPtr<Gio::AsyncResult>&
     }
     else
     {
-      const bool is_first_item = m_iter_new_template_dummy.get() != NULL;
+      const bool is_first_item = m_iter_new_template_dummy.get();
 
       // Add to list
       Gtk::TreeModel::iterator iter = m_new_model->append(m_iter_new_template->children());
@@ -785,6 +927,7 @@ void Dialog_ExistingOrNew::on_existing_button_clicked(const Gtk::TreeModel::Path
     on_select_clicked();
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void Dialog_ExistingOrNew::on_new_row_activated(const Gtk::TreeModel::Path& /* path */, Gtk::TreeViewColumn* /* column */)
 {
   if(m_select_button->is_sensitive())
@@ -798,6 +941,7 @@ void Dialog_ExistingOrNew::on_new_button_clicked(const Gtk::TreeModel::Path& pat
   if(m_select_button->is_sensitive())
     on_select_clicked();
 }
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
 void Dialog_ExistingOrNew::on_select_clicked()
 {
@@ -806,17 +950,22 @@ void Dialog_ExistingOrNew::on_select_clicked()
 
   if(action == OPEN_URI && iter == m_iter_existing_other)
   {
+    #ifdef GLOM_ENABLE_MAEMO
+    Hildon::FileChooserDialog dialog(Gtk::FILE_CHOOSER_ACTION_OPEN);
+    #else
     Gtk::FileChooserDialog dialog(*this, "Choose a glom file to open");
+    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
+    dialog.set_default_response(Gtk::RESPONSE_OK);
+    #endif // GLOM_ENABLE_MAEMO
+
     Gtk::FileFilter filter;
     filter.add_mime_type("application/x-glom");
     filter.set_name("Glom files");
     dialog.add_filter(filter);
 
-    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
-    //dialog.set_default_response(Gtk::RESPONSE_ACCEPT);
-
-    if(dialog.run() == Gtk::RESPONSE_ACCEPT)
+    const int response_id = dialog.run();
+    if(response_id == Gtk::RESPONSE_OK)
     {
       m_chosen_uri = dialog.get_uri();
       response(Gtk::RESPONSE_ACCEPT);
