@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libglom/libglom_config.h> // For GLOM_ENABLE_MAEMO
+#include <libglom/libglom_config.h>
  
 #include <libglom/connectionpool.h>
 #include <libglom/document/document.h>
@@ -26,25 +26,18 @@
 //#include <libgdamm/connectionevent.h>
 #include <glibmm/i18n.h>
 
-#ifndef G_OS_WIN32
+#ifdef G_OS_WIN32
+# include <windows.h>
+#else
 # include <libepc/shell.h> //For epc_shell_set_progress_hooks().
 # include <libepc/publisher.h>
-#else
-// objidl.h, included by windows.h, defines a type called DATADIR, so we need
-// to undef it temporarily.
-# define GLOM_SAVE_DATADIR DATADIR
-# undef DATADIR
-# include <windows.h>
-# define DATADIR GLOM_SAVE_DATADIR
 #endif
 
 #include <signal.h> //To catch segfaults
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
 static EpcProtocol publish_protocol = EPC_PROTOCOL_HTTPS;
 #endif
-#endif
 
 // Uncomment to see debug messages
 //#define GLOM_CONNECTION_DEBUG
@@ -208,10 +201,8 @@ ConnectionPool* ConnectionPool::m_instance = 0;
 
 ConnectionPool::ConnectionPool()
 :
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   m_epc_publisher(0),
   m_dialog_epc_progress(0),
-#endif // !GLOM_ENABLE_CLIENT_ONLY
   m_backend(0),
   m_sharedconnection_refcount(0),
   m_ready_to_connect(false),
@@ -293,7 +284,7 @@ sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<Except
   if(!(connection_pool->m_backend.get()))
   {
     std::cerr << "ConnectionPool::get_and_connect(): m_backend is null." << std::endl;
-    return result; //TODO: Return a FAILURE_NO_BACKEND erro?, though that would be tedious.
+    return result; //TODO: Return a FAILURE_NO_BACKEND error?, though that would be tedious.
   }
   
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
@@ -385,11 +376,25 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
       {
         //Allow get_meta_store_data() to succeed:
         //Hopefully this (and the update_meta_store_for_table() calls) is all we need.
-        std::cout << "DEBUG: Calling update_meta_store_data_types() ..." << std::endl;
-        m_refGdaConnection->update_meta_store_data_types();
-        std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
-
-        std::cout << "DEBUG: Calling update_meta_store_table_names() ..." << std::endl;
+        //std::cout << "DEBUG: Calling update_meta_store_data_types() ..." << std::endl;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+        try
+        {
+          m_refGdaConnection->update_meta_store_data_types();
+        }
+        catch(const Glib::Error& ex)
+        {
+          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex.what() << std::endl;
+        }
+        //std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
+#else
+        std::auto_ptr<Glib::Error> ex;
+        m_refGdaConnection->update_meta_store_data_types(ex);
+        if (ex.get())
+          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
+#endif
+        //std::cout << "DEBUG: Calling update_meta_store_table_names() ..." << std::endl;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
         try
         {
           //update_meta_store_table_names() has been known to throw an exception.
@@ -398,16 +403,20 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << "update_meta_store_table_names() failed: " << ex.what() << std::endl;
+          std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
         }
-        std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
+        //std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
+#else
+        m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name(), ex);
+        if (ex.get())
+          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
+#endif
 
         // Connection succeeded
         // Create the fieldtypes member if it has not already been done:
         if(!m_pFieldTypes)
           m_pFieldTypes = new FieldTypes(m_refGdaConnection);
           
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
         //Let other clients discover this server via avahi:
         //TODO: Only advertize if we are the first to open the document,
@@ -418,7 +427,6 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
         if(document && document->get_network_shared())
           avahi_start_publishing(); //Stopped in the signal_finished handler.
 #endif // !G_OS_WIN32
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
         return connect(); //Call this method recursively. This time m_refGdaConnection exists.
@@ -436,7 +444,6 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
   return sharedptr<SharedConnection>(0);
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
 void ConnectionPool::create_database(const Glib::ustring& database_name)
 #else
@@ -453,7 +460,6 @@ void ConnectionPool::create_database(const Glib::ustring& database_name, std::au
   if(error.get()) throw *error;
 #endif
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void ConnectionPool::set_user(const Glib::ustring& value)
 {
@@ -544,11 +550,9 @@ void ConnectionPool::on_sharedconnection_finished()
 
     m_refGdaConnection.reset();
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
     //TODO: this should only even be started if we are the first to open the .glom file:
     avahi_stop_publishing();
-#endif
 #endif
 
     //g_warning("ConnectionPool: connection closed");
@@ -682,7 +686,6 @@ bool ConnectionPool::set_network_shared(const SlotProgress& slot_progress, bool
     return false;
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
 bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field)
 #else
@@ -704,19 +707,20 @@ bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr
 
   if(!m_refGdaConnection)
     return false;
-
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   std::auto_ptr<Glib::Error> error;
+#endif
   const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   if(error.get()) throw *error;
-
   m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
 #else
-  const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
-  if(result)
-    result = m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl;   
 #endif
-
   return result;
 }
 
@@ -744,16 +748,18 @@ bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::us
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   std::auto_ptr<Glib::Error> error;
+#endif
   const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   if(error.get()) throw *error;
-
   m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
 #else
-  const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
-  if(result)
-    result = m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
 #endif
-
   return result;
 }
 
@@ -797,19 +803,19 @@ bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   std::auto_ptr<Glib::Error> error;
-  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
-  if(error.get()) throw *error;
-
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-#else
-  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
-  if(result)
-    result = m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
 #endif
+  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  if(error.get()) throw *error;
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
 
+#else
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+#endif
   return result;
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 ConnectionPool::InitErrors ConnectionPool::initialize(const SlotProgress& slot_progress, bool network_shared)
 {
@@ -819,7 +825,6 @@ ConnectionPool::InitErrors ConnectionPool::initialize(const SlotProgress& slot_p
     return Backend::INITERROR_OTHER;
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 Document* ConnectionPool::get_document()
 {
   if(!m_slot_get_document)
@@ -830,9 +835,7 @@ Document* ConnectionPool::get_document()
 
   return m_slot_get_document();
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
 //static
 EpcContents* ConnectionPool::on_publisher_document_requested(EpcPublisher* /* publisher */, const gchar* /* key */, gpointer user_data)
@@ -1002,7 +1005,5 @@ void ConnectionPool::set_get_document_func(const SlotGetDocument& slot)
   m_slot_get_document = slot;
 }
 
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
 
 } //namespace Glom
