@@ -21,12 +21,11 @@
 #ifndef GLOM_DATASTRUCTURE_GLOMCONVERSIONS_H
 #define GLOM_DATASTRUCTURE_GLOMCONVERSIONS_H
 
-#include <glom/libglom/data_structure/field.h>
-#include <glom/libglom/data_structure/numeric_format.h>
+#include <libglom/data_structure/field.h>
+#include <libglom/data_structure/numeric_format.h>
 
-#include <glom/libglom/data_structure/layout/layoutitem_field.h>
+#include <libglom/data_structure/layout/layoutitem_field.h>
 #include <libxml++/libxml++.h>
-#include <gdkmm/pixbuf.h>
 
 namespace Glom
 {
@@ -46,7 +45,7 @@ namespace Conversions
   Glib::ustring format_date(const tm& tm_data);
   Glib::ustring format_date(const tm& tm_data, const std::locale& locale, bool iso_format = false);
 
-  Gnome::Gda::Value parse_value(double number);
+  Gnome::Gda::Value parse_value(double number);  
   Gnome::Gda::Value parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, bool& success, bool iso_format = false);
   Gnome::Gda::Value parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, const NumericFormat& numeric_format, bool& success, bool iso_format = false);
 
@@ -70,7 +69,7 @@ namespace Conversions
    *
    * @result true if 4 digits are used.
    */
-  bool sanity_check_date_text_representation_uses_4_digit_years();
+  bool sanity_check_date_text_representation_uses_4_digit_years(bool debug_output = false);
 
   Glib::ustring format_tm(const tm& tm_data, const std::locale& locale, const char* format);
   //static tm parse_tm(const Glib::ustring& text, const std::locale& locale, char format);
@@ -80,22 +79,9 @@ namespace Conversions
 
   Gnome::Gda::Value get_example_value(Field::glom_field_type field_type);
 
-  ///Get a string representing binary data, for SQL
-  Glib::ustring escape_binary_data_postgres(guint8* buffer, size_t buffer_size);
-  Glib::ustring escape_binary_data_sqlite(guint8* buffer, size_t buffer_size);
-  guint8* unescape_binary_data_postgres(const Glib::ustring& escaped_binary_data, size_t& length);
-  guint8* unescape_binary_data_sqlite(const Glib::ustring& escaped_binary_data, size_t& length);
-
   Gnome::Gda::Value convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type);
-
-
-  Glib::RefPtr<Gdk::Pixbuf> get_pixbuf_for_gda_value(const Gnome::Gda::Value& value);
-}
-
-//Copied from Postgres's PQunescapeBytea() so I don't have the trouble of finding and linking to the 
-//postgres libraries directly, without the benefit of a pkg-config .pc file. murrayc.
-unsigned char *
-Glom_PQunescapeBytea(const unsigned char *strtext, size_t *retbuflen);
+  
+} //namespace Conversions
 
 } //namespace Glom
 
