@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libglom/libglom_config.h> // For GLOM_ENABLE_MAEMO
+#include <libglom/libglom_config.h>
 
 #include <libglom/utils.h>
 #include <libglom/connectionpool.h>
@@ -29,10 +29,6 @@
 
 #include <giomm.h>
 
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/note.h>
-#endif
-
 #include <string.h> // for strchr
 #include <sstream> //For stringstream
 
@@ -138,14 +134,28 @@ Glib::ustring Utils::trim_whitespace(const Glib::ustring& text)
 
 Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with)
 {
+  if(search_for.empty())
+  {
+    std::cerr << "Utils::string_replace(): search_for was empty." << std::endl;
+    return src;
+  }
+
+  //std::cout << "debug: Utils::string_replace(): src=" << src << ", search_for=" << search_for << ", replace_with=" << replace_with << std::endl;
+
   std::string result = src;
 
   std::string::size_type pos = 0;
-  std::string::size_type len= search_for.size();
+  const std::string::size_type len_search = search_for.size();
+  const std::string::size_type len_replace = replace_with.size();
 
-  while((pos = result.find(search_for)) != std::string::npos)
+  std::string::size_type pos_after_prev = 0;
+  while((pos = result.find(search_for, pos_after_prev)) != std::string::npos)
   {
-   result.replace(pos, len, replace_with);
+    //std::cout << "  debug: pos=" << pos << ", found=" << search_for << ", in string: " << result.substr(pos_after_prev, 20) << std::endl;
+    //std::cout << "  debug: before: result =" << result << ", pos_after_prev=pos_after_prev" << std::endl;
+    result.replace(pos, len_search, replace_with);
+    //std::cout << "  after: before: result = result" << std::endl;
+    pos_after_prev = pos + len_replace;
   }
 
   return result;
@@ -227,9 +237,12 @@ static void add_to_relationships_list(type_list_relationships& list_relationship
  
 }
 
-Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& extra_join, const type_sort_clause& sort_clause, const Glib::ustring& extra_group_by)
+
+Glib::ustring Utils::build_sql_select_fields_to_get(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const type_sort_clause& sort_clause, Glib::ustring& sql_part_from, Glib::ustring& sql_part_leftouterjoin)
 {
-  Glib::ustring result;
+  //Initialize output parameters:
+  sql_part_from = Glib::ustring();
+  sql_part_leftouterjoin = Glib::ustring();
 
   //Get all relationships used in the query:
   typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
@@ -249,7 +262,6 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
 
 
   Glib::ustring sql_part_fields;
-  Glib::ustring sql_part_from;
 
   for(type_vecConstLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
@@ -284,17 +296,12 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
 
   if(sql_part_fields.empty())
   {
-    std::cerr << "Utils::build_sql_select_with_where_clause(): sql_part_fields.empty(): fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
-    return result;
-  }
-  else
-  {
-    result =  "SELECT " + sql_part_fields +
-      " FROM \"" + table_name + "\"";
+    std::cerr << "Utils::build_sql_select_fields_to_get(): sql_part_fields.empty(): fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
+    return sql_part_fields;
   }
 
-  //LEFT OUTER JOIN will get the field values from the other tables, and give us our fields for this table even if there is no corresponding value in the other table.
-  Glib::ustring sql_part_leftouterjoin; 
+  //LEFT OUTER JOIN will get the field values from the other tables, 
+  //and give us our fields for this table even if there is no corresponding value in the other table.
   for(type_list_relationships::const_iterator iter = list_relationships.begin(); iter != list_relationships.end(); ++iter)
   {
     sharedptr<const UsesRelationship> uses_relationship = *iter;
@@ -313,17 +320,34 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
     }
   }
 
-  if(!extra_join.empty())
-  {
-    sql_part_leftouterjoin += (" " + extra_join + " ");
-  }
+  return sql_part_fields;
+}
+
+
+Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& extra_join, const type_sort_clause& sort_clause, const Glib::ustring& extra_group_by)
+{
+  //Get the list of fields to SELECT, plus the tables that they are selected FROM.
+  Glib::ustring sql_part_from;
+  Glib::ustring sql_part_leftouterjoin;
+  const Glib::ustring sql_part_fields = Utils::build_sql_select_fields_to_get(
+    table_name, fieldsToGet, sort_clause, sql_part_from, sql_part_leftouterjoin);
+
+  //Build the whole SQL statement:
+  Glib::ustring result = 
+    "SELECT " + sql_part_fields +
+    " FROM \"" + table_name + "\"";
 
   if(!sql_part_from.empty())
     result += ("," + sql_part_from);
 
+  if(!extra_join.empty())
+    sql_part_leftouterjoin += (" " + extra_join + " ");
+
   if(!sql_part_leftouterjoin.empty())
     result += (" " + sql_part_leftouterjoin);
 
+
+  //Add the WHERE clause:
   if(!where_clause.empty())
     result += " WHERE " + where_clause;
 
@@ -434,7 +458,7 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
   Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->statement_execute_select(sql_query);
 #else
   std::auto_ptr<Glib::Error> error;
-  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->statement_execute_select(sql_query, error);
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->statement_execute_select(sql_query, Gnome::Gda::STATEMENT_MODEL_RANDOM_ACCESS, error);
 #endif
 
   if(datamodel)
@@ -443,13 +467,21 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
     //std::cout << "  result: count=" << count << std::endl;
     for(guint row = 0; row < count; ++row)
     {
+
       std::pair<Gnome::Gda::Value, Gnome::Gda::Value> itempair;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       itempair.first = datamodel->get_value_at(0, row);
 
       if(with_second)
         itempair.second = datamodel->get_value_at(1, row);
+#else
+      itempair.first = datamodel->get_value_at(0, row, error);
 
-      list_values.push_back(itempair);
+      if(with_second)
+        itempair.second = datamodel->get_value_at(1, row, error);
+#endif
+
+      list_values.push_back(itempair);      
     }
   }
   else
@@ -793,8 +825,7 @@ bool Utils::file_exists(const Glib::ustring& uri)
       return false; //Something went wrong. It does not exist.
     }
 #else
-      std::auto_ptr<Gio::Error> error;
-      retrun file->query_exists(error);
+      return file->query_exists();
 #endif
   }
 }
