@@ -21,11 +21,11 @@
 #ifndef BOX_DATA_H
 #define BOX_DATA_H
 
-#include <libglom/libglom_config.h> // GLOM_ENABLE_CLIENT_ONLY
+#include "config.h" // GLOM_ENABLE_CLIENT_ONLY
 
 #include <glom/box_withbuttons.h>
 #include <glom/base_db_table_data.h>
-#include "dialog_layout.h"
+#include <glom/mode_design/layout/dialog_layout.h>
 
 namespace Glom
 {
