@@ -18,11 +18,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "box_data_portal.h"
-#include <glom/libglom/data_structure/glomconversions.h>
-#include <glom/libglom/glade_utils.h>
+#include <glom/mode_data/box_data_portal.h>
+#include <libglom/data_structure/glomconversions.h>
+#include <glom/glade_utils.h>
 #include <glom/frame_glom.h> //For show_ok_dialog()
-#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <glom/utils_ui.h> //For bold_message()).
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -86,7 +86,7 @@ bool Box_Data_Portal::init_db_details(const Glib::ustring& parent_table, bool sh
       relationship_title = _("Undefined Table");
     }
 
-    m_Label.set_markup(Bakery::App_Gtk::util_bold_message(relationship_title));
+    m_Label.set_markup(Utils::bold_message(relationship_title));
     m_Label.show();
 
     m_Alignment.set_padding(Utils::DEFAULT_SPACING_SMALL /* top */, 0, Utils::DEFAULT_SPACING_LARGE /* left */, 0);
@@ -162,10 +162,10 @@ void Box_Data_Portal::on_record_added(const Gnome::Gda::Value& /* primary_key_va
 
 Box_Data_Portal::type_vecLayoutFields Box_Data_Portal::get_fields_to_show() const
 {
-  const Document_Glom* document = get_document();
+  const Document* document = get_document();
   if(document && m_portal)
   {
-    Document_Glom::type_list_layout_groups mapGroups;
+    Document::type_list_layout_groups mapGroups;
     mapGroups.push_back(m_portal);
 
     sharedptr<const Relationship> relationship = m_portal->get_relationship();
@@ -211,48 +211,50 @@ bool Box_Data_Portal::get_has_suitable_record_to_view_details() const
 
 void Box_Data_Portal::get_suitable_table_to_view_details(Glib::ustring& table_name, sharedptr<const UsesRelationship>& relationship) const
 {
- 
   //Initialize output parameters:
   table_name = Glib::ustring();
 
   if(!m_portal)
     return;
 
+
+  sharedptr<const UsesRelationship> navigation_relationship = m_portal->get_navigation_relationship_specific();
+
   //Check whether a relationship was specified:
-  bool navigation_relationship_main = false;
-  sharedptr<const UsesRelationship> navigation_relationship = m_portal->get_navigation_relationship_specific(navigation_relationship_main);
-  if(!navigation_relationship_main && !navigation_relationship)
+  if(m_portal->get_navigation_type() == LayoutItem_Portal::NAVIGATION_AUTOMATIC)
   {
     //std::cout << "debug: decide automatically." << std::endl;
     //Decide automatically:
-    navigation_relationship_main = false;
+    bool navigation_relationship_main = false; // no longer used
     navigation_relationship = get_portal_navigation_relationship_automatic(m_portal, navigation_relationship_main);
     //std::cout << "debug: auto main=" << navigation_relationship_main << ", navigation_relationship=" << (navigation_relationship ? navigation_relationship->get_name() : navigation_relationship->get_relationship()->get_name()) << std::endl;
   }
   //else
   //  std::cout << "debug: get_suitable_table_to_view_details(): Using specific nav." << std::endl;
 
-  const Document_Glom* document = get_document();
+  const Document* document = get_document();
   if(!document)
     return;
 
 
   //Get the navigation table name from the chosen relationship:
   const Glib::ustring directly_related_table_name = m_portal->get_table_used(Glib::ustring() /* not relevant */);
- 
+
+  // The navigation_table_name (and therefore, the table_name output parameter,
+  // as well) stays empty if the navrel type was set to none.
   Glib::ustring navigation_table_name;
-  if(navigation_relationship_main)
+  if(m_portal->get_navigation_type() == LayoutItem_Portal::NAVIGATION_AUTOMATIC)
   {
     navigation_table_name = directly_related_table_name;
   }
-  else if(navigation_relationship)
+  else if(m_portal->get_navigation_type() == LayoutItem_Portal::NAVIGATION_SPECIFIC)
   {
     navigation_table_name = navigation_relationship->get_table_used(directly_related_table_name);
   }
 
   if(navigation_table_name.empty())
   {
-    std::cerr << "Box_Data_Portal::get_suitable_table_to_view_details(): navigation_table_name is empty." << std::endl;
+    //std::cerr << "Box_Data_Portal::get_suitable_table_to_view_details(): navigation_table_name is empty." << std::endl;
     return;
   }
 
@@ -297,7 +299,12 @@ void Box_Data_Portal::get_suitable_record_to_view_details(const Gnome::Gda::Valu
   type_vecLayoutFields fieldsToGet;
   fieldsToGet.push_back(layout_item);
 
-  const Glib::ustring query = Utils::build_sql_select_with_key(m_portal->get_table_used(Glib::ustring() /* not relevant */), fieldsToGet, get_key_field(), primary_key_value);
+  //For instance "invoice_line_id" if this is a portal to an "invoice_lines" table:
+  const Glib::ustring related_table = m_portal->get_table_used(Glib::ustring() /* not relevant */);
+  sharedptr<const Field> key_field = get_field_primary_key_for_table(related_table);
+  //std::cout << "DEBUG: related table=" << related_table << ", whose primary_key=" << key_field->get_name() << std::endl;
+
+  const Glib::ustring query = Utils::build_sql_select_with_key(related_table, fieldsToGet, key_field, primary_key_value);
   Glib::RefPtr<Gnome::Gda::DataModel> data_model = query_execute_select(query);
 
 
@@ -306,8 +313,12 @@ void Box_Data_Portal::get_suitable_record_to_view_details(const Gnome::Gda::Valu
   {
     //Set the output parameters:
     table_name = navigation_table_name;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED    
     table_primary_key_value = data_model->get_value_at(0, 0);
-
+#else
+    std::auto_ptr<Glib::Error> error;    
+    table_primary_key_value = data_model->get_value_at(0, 0, error);
+#endif    
     //std::cout << "Box_Data_Portal::get_suitable_record_to_view_details(): table_primary_key_value=" << table_primary_key_value.to_string() << std::endl;
 
     //The value is empty when there there is no record to match the key in the related table:
@@ -321,7 +332,14 @@ void Box_Data_Portal::get_suitable_record_to_view_details(const Gnome::Gda::Valu
   else
   {
     value_found = false;
-    std::cout << "debug: Box_Data_Portal::get_suitable_record_to_view_details(): SQL query returned no suitable primary key." << std::endl;
+
+    std::cout << "DEBUG: Box_Data_Portal::get_suitable_record_to_view_details(): SQL query returned no suitable primary key. table=" 
+      << related_table  
+      << ", field=" << layout_item->get_layout_display_name() 
+      << ", key_field=" << key_field->get_name()
+      << ", primary_key_value=" << primary_key_value.to_string() << std::endl;
+
+    std::cout << "  DEBUG: SQL was: " << query << std::endl;
   }
 
   if(!value_found)
@@ -336,9 +354,9 @@ void Box_Data_Portal::get_suitable_record_to_view_details(const Gnome::Gda::Valu
   }
 }
 
-Document_Glom::type_list_layout_groups Box_Data_Portal::create_layout_get_layout()
+Document::type_list_layout_groups Box_Data_Portal::create_layout_get_layout()
 {
-  Document_Glom::type_list_layout_groups result;
+  Document::type_list_layout_groups result;
 
   if(m_portal)
     result.push_back(m_portal);
