@@ -24,17 +24,22 @@
 #include "dialog_design.h"
 #include "fields/box_db_table_definition.h"
 
+namespace Glom
+{
+
 class Dialog_Fields : public Dialog_Design
 {
 public:
-  Dialog_Fields(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
+  Dialog_Fields(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Dialog_Fields();
 
-  virtual void init_db_details(const Glib::ustring& strTableName);
-    
-protected:
+  virtual bool init_db_details(const Glib::ustring& table_name);
+
+private:
 
   Box_DB_Table_Definition* m_box;
 };
 
+} //namespace Glom
+
 #endif //DIALOG_FIELDS_H
