@@ -18,53 +18,54 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 //We need to include this before anything else, to avoid redefinitions:
 #include <Python.h>
 #include <compile.h> /* for the PyCodeObject */
 #include <eval.h> /* for PyEval_EvalCode */
 #include <objimpl.h> /* for PyObject_New() */
 
-#include <glom/libglom/python_embed/py_glom_record.h>
-#include <glom/libglom/python_embed/py_glom_related.h>
-#include <glom/libglom/python_embed/py_glom_relatedrecord.h>
+#include <libglom/python_embed/py_glom_record.h>
+#include <libglom/python_embed/py_glom_related.h>
+#include <libglom/python_embed/py_glom_relatedrecord.h>
 
 static PyMethodDef pyglom_methods[] = {
     {NULL, 0, 0, 0}  /* Sentinel */
 };
 
 PyMODINIT_FUNC
-initglom(void) 
+initglom_1_12(void) 
 {
   PyObject* m;
 
   //pyglom_RecordType.tp_new = PyType_GenericNew;
 
-  if(PyType_Ready(PyGlomRecord_GetPyType()) < 0)
+  if(PyType_Ready(Glom::PyGlomRecord_GetPyType()) < 0)
     return;
 
-  if(PyType_Ready(PyGlomRelated_GetPyType()) < 0)
+  if(PyType_Ready(Glom::PyGlomRelated_GetPyType()) < 0)
     return;
 
-  if(PyType_Ready(PyGlomRelatedRecord_GetPyType()) < 0)
+  if(PyType_Ready(Glom::PyGlomRelatedRecord_GetPyType()) < 0)
     return;
 
 
-  m = Py_InitModule3("glom", pyglom_methods,
-                      "Python module for Glom caluclated fields.");
+  m = Py_InitModule3((char*)"glom_" GLOM_ABI_VERSION_UNDERLINED, pyglom_methods,
+                      (char*)"Python module for Glom caluclated fields.");
 
 
-  Py_INCREF(PyGlomRecord_GetPyType());
-  PyModule_AddObject(m, "Record", (PyObject *)PyGlomRecord_GetPyType());
+  Py_INCREF(Glom::PyGlomRecord_GetPyType());
+  PyModule_AddObject(m, (char*)"Record", (PyObject *)Glom::PyGlomRecord_GetPyType());
 
-  Py_INCREF(PyGlomRelated_GetPyType());
-  PyModule_AddObject(m, "Related", (PyObject *)PyGlomRelated_GetPyType());
+  Py_INCREF(Glom::PyGlomRelated_GetPyType());
+  PyModule_AddObject(m, (char*)"Related", (PyObject *)Glom::PyGlomRelated_GetPyType());
 
-  Py_INCREF(PyGlomRelated_GetPyType());
-  PyModule_AddObject(m, "RelatedRecord", (PyObject *)PyGlomRelated_GetPyType());
+  Py_INCREF(Glom::PyGlomRelated_GetPyType());
+  PyModule_AddObject(m, (char*)"RelatedRecord", (PyObject *)Glom::PyGlomRelated_GetPyType());
 
 
   if(PyErr_Occurred())
-    Py_FatalError ("Can't initialise glom module");
+    Py_FatalError((char*)"Can't initialise glom module");
 }
 
 
